@@ -23,9 +23,13 @@ sub import {
 sub AUTOLOAD {
   my ($name) = our $AUTOLOAD =~ /::(\w+)$/;
   die "FATAL: unknown method '$name'" unless $name =~ /^[A-Z0-9_]+$/;
-  splice(@_, 1, 0, $name);
-  goto &SetAttribute if scalar(@_)>2;
-  goto &GetAttribute;
+  my $method = sub {
+        return $_[0]->GetAttribute($name) if scalar(@_) == 1; 
+        return $_[0]->SetAttribute($name, $_[1]) if scalar(@_) > 1;
+  };
+  no strict 'refs';
+  *{$AUTOLOAD} = $method;
+  goto &$method;
 }
 
 sub BEGIN {
@@ -139,13 +143,14 @@ sub SetAttribute {
   #BEWARE: we need to keep the order of attribute assignment - thus cannot use for (keys %args) {...}
   while(1) {    
     my $k = shift;    
-    last unless defined $k;
+    carp("Warning: invalid attribute name"), last unless defined $k;
     my $v = shift;
     if (!ref($v)) {      
       IUP::Internal::LibraryIup::_IupStoreAttribute($self->ihandle, $k, $v);
     }
     elsif (blessed($v) && $v->can('ihandle')) {
-      #carp "Debug: attribute '$k' is a refference '" . ref($v) . "'";
+warn "XXX: setting handle to attr '$k' for ", ref($v), "\n";
+    #carp "Debug: attribute '$k' is a refference '" . ref($v) . "'";
       IUP::Internal::LibraryIup::_IupSetAttributeHandle($self->ihandle, $k, $v->ihandle);      
       #assuming any element ref stored into iup attribute to be a child      
       unless($self->_get_child_ref($v)) {
@@ -158,6 +163,7 @@ sub SetAttribute {
     else {
       carp "[warning] cannot set attribute '$k' to '$v'";
     }
+    last unless @_;
   }
 }
 
